/**
 * Copyright (C) 2014 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.httpinvoker;

import java.io.File;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.net.URL;
import java.util.Map;
import java.util.Properties;

import net.sf.ehcache.CacheManager;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Platform;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.osgi.framework.Bundle;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.context.SecurityContextImpl;
import org.unitils.UnitilsJUnit4;
import org.unitils.database.DatabaseUnitils;
import org.unitils.database.annotations.Transactional;
import org.unitils.database.util.TransactionMode;
import org.unitils.spring.annotation.SpringApplicationContext;

import eu.etaxonomy.cdm.api.application.CdmApplicationRemoteController;
import eu.etaxonomy.cdm.api.application.CdmApplicationState;
import eu.etaxonomy.cdm.api.application.CdmDataChangeService;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.remoting.cache.CdmRemoteCacheManager;
import eu.etaxonomy.taxeditor.remoting.cache.CdmTransientEntityCacher;
import eu.etaxonomy.taxeditor.remoting.source.CdmPersistentRemoteSource;
import eu.etaxonomy.taxeditor.remoting.source.CdmRemoteSource;
import eu.etaxonomy.taxeditor.remoting.source.CdmRemoteSourceException;
import eu.etaxonomy.taxeditor.remoting.source.ICdmRemoteSource;
import eu.etaxonomy.taxeditor.session.CdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionManager;


/**
 * Base class for remoting tests, responsible for
 * - starting / stop the cdm server
 * - running some basic connection tests
 * - setting up the remote configuration.
 *
 */
@Transactional(TransactionMode.DISABLED)
@SpringApplicationContext("file:./target/classes/eu/etaxonomy/cdm/testRemotingApplicationContext.xml")
public abstract class BaseRemotingTest extends UnitilsJUnit4 {
    private static final Logger logger = Logger.getLogger(BaseRemotingTest.class);


    public static final Resource SERVER_PROPERTIES_FILE =
            new ClassPathResource("server.properties");

    public static final Resource EDITOR_DATASOURCES_FILE =
            new ClassPathResource(".cdmLibrary/writableResources/cdm.datasources.xml");

    //private static CdmApplicationRemoteController remoteApplicationController;
    private static ICdmRemoteSource cdmRemoteSource;
    private static CdmPersistentRemoteSource remotePersistentSource;

    public static boolean useManagedServer = true;


    protected static ICdmEntitySessionManager cdmEntitySessionManager;

    private final static String DEFAULT_USER = "admin";
    private final static String DEFAULT_PASSWORD = "00000";

    private static String userHomeKey = "user.home";

    private static String user = DEFAULT_USER;
    private static String password = DEFAULT_PASSWORD;

    @BeforeClass
    public static void initializeBaseRemotingTest() {

        //NOTE: Run this the cdmTest H2 DB whenever it needs to be
        //      recreated e.g. after a model change
        DatabaseUnitils.disableConstraints();

        try {
            String userHomeDirPath;
            Bundle bundle = Platform.getBundle("eu.etaxonomy.taxeditor.test");
            // if bundle is null then we are running as junit test in eclipse
            if(bundle == null) {
                userHomeDirPath = EDITOR_DATASOURCES_FILE.getFile().getParentFile().getParentFile().getParent();
            } else {
                // ... else we are running in maven which requires loading the dir via the
                // classpath bundle
                URL userHomeDirURL = bundle.getEntry("src/test/resources");
                File userHomeDir = new File(FileLocator.resolve(userHomeDirURL).toURI());
                userHomeDirPath = userHomeDir.getAbsolutePath();
            }


            logger.info("Setting user.home to " + userHomeDirPath);
            System.setProperty(userHomeKey, userHomeDirPath);

            CDMServer cdmServer = CDMServer.getInstance();


            Properties prop = new Properties();
            InputStream inputStream = SERVER_PROPERTIES_FILE.getInputStream();

            if (inputStream != null) {
                prop.load(inputStream);
                inputStream.close();
            }

            if(prop.getProperty("httpPort") != null) {
                cdmServer.setHttpPort(Integer.valueOf(prop.getProperty("httpPort")));
            }

            if(prop.getProperty("stopPort") != null) {
                cdmServer.setStopPort(Integer.valueOf(prop.getProperty("stopPort")));
            }

            if(prop.getProperty("stopKey") != null) {
                cdmServer.setStopKey(prop.getProperty("stopKey"));
            }

            if(prop.getProperty("user") != null) {
                user = prop.getProperty("user");
            }

            if(prop.getProperty("password") != null) {
                password = prop.getProperty("password");
            }

            cdmServer.start();
            initializeController(CDMServer.getInstance().getName(),
                    CDMServer.getInstance().getHost(),
                    CDMServer.getInstance().getPort(),
                    CDMServer.getInstance().getContextPath(),
                    NomenclaturalCode.ICNAFP,
                    user,
                    password);
        } catch (Exception e) {
            e.printStackTrace();
           // Assert.fail("Server failed to start. Reason : " + e.getMessage());
        }

        logger.info("Emptying all caches (except model cache) ");
        //emptyAllCachesExceptModelCache();
    }

    public static void emptyAllCachesExceptModelCache() {
        CacheManager cm = CacheManager.create();
        String[] cacheNames = CacheManager.create().getCacheNames();
        for(String cacheName : cacheNames) {
            if(!cacheName.equals(CdmRemoteCacheManager.CDM_MODEL_CACHE_NAME)) {
                cm.removeCache(cacheName);
            }
        }
    }


    public static void initializeController(String sourceName, String host, int port, String contextPath, NomenclaturalCode ncode, String username, String password) {

        if(CdmApplicationState.getCurrentAppConfig() != null) {
            return;
        }
        cdmRemoteSource = CdmRemoteSource.NewInstance(sourceName, host, port, contextPath, ncode);
        CdmApplicationRemoteController remoteApplicationController =
                CdmApplicationRemoteController.NewInstance(cdmRemoteSource,
                        null,
                        null);
        CdmApplicationState.setCurrentAppConfig(remoteApplicationController);
        try {
            remotePersistentSource = CdmPersistentRemoteSource.NewInstance(sourceName);
        } catch (CdmRemoteSourceException e) {
            Assert.fail("Default Remote Persistent Source failed to load. Reason : " + e.getMessage());
        }
        cdmEntitySessionManager = getRemoteApplicationController().getCdmEntitySessionManager();

        CdmApplicationState.setCurrentDataChangeService(new CdmDataChangeService());

        //FIXME:Remoting the authentication code should be replaced by a method call which actually
        // does the authentication in the editor code so that the 'real' authentication can be tested
        SecurityContextHolder.clearContext();
        SecurityContextImpl sc = new SecurityContextImpl();
        Authentication token = new UsernamePasswordAuthenticationToken(username,password);
        Authentication authentication = getRemoteApplicationController().getAuthenticationManager().authenticate(token);
        authentication = new UsernamePasswordAuthenticationToken(username,password, authentication.getAuthorities());
        sc.setAuthentication(authentication);

        SecurityContextHolder.setContext(sc);
    }



    protected static CdmApplicationRemoteController getRemoteApplicationController() {
        return (CdmApplicationRemoteController) CdmApplicationState.getCurrentAppConfig();
    }

    protected static ICdmRemoteSource getCdmRemoteSource() {
        return cdmRemoteSource;
    }

    protected static CdmPersistentRemoteSource getCdmPersistentRemoteSource() {
        return remotePersistentSource;
    }

    protected static ICdmEntitySessionManager getCdmEntitySessionManager() {
        return cdmEntitySessionManager;
    }


    protected static CdmEntitySession getSession(ICdmEntitySessionEnabled sessionOwner) {
        Map<ICdmEntitySessionEnabled, CdmEntitySession> ownerSessionMap =
                (Map<ICdmEntitySessionEnabled, CdmEntitySession>) getFieldValueViaReflection(cdmEntitySessionManager, "ownerSessionMap");
        return ownerSessionMap.get(sessionOwner);
    }

    protected static CdmEntitySession getActiveSession() {
        return (CdmEntitySession) getFieldValueViaReflection(cdmEntitySessionManager, "activeSession");
    }

    protected static CdmTransientEntityCacher getCacher(ICdmEntitySessionEnabled sessionOwner) {
        return (CdmTransientEntityCacher) getFieldValueViaReflection(getSession(sessionOwner), "cdmTransientEntityCacher");
    }

    @AfterClass
    public static void cleanup() {
        try {
            CDMServer.getInstance().stop();
        } catch (Exception e) {
            Assert.fail("Server could not be stopped. Reason : " + e.getMessage());
        }
    }


    protected static Object getFieldValueViaReflection(Object object, String fieldName) {
        Class<?> clazz = object.getClass();
        try {
            Field field = clazz.getDeclaredField(fieldName);
            field.setAccessible(true);
            return field.get(object);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return null;
    }




}
